//----------------------------------------------------------------------------
//  Project QuickFyre
//  The BRiGADE
//  Copyright  1997. All Rights Reserved.
//
//  SUBSYSTEM:    QuickFyre Application
//  FILE:         quickfyredlgclient.cpp
//  AUTHOR:       Pri$m
//
//  OVERVIEW
//  ~~~~~~~~
//  Source file for implementation of TQuickFyreDlgClient (TDialog).
//
//----------------------------------------------------------------------------

#include <owl/pch.h>
#include <owl/dc.h>
#include <io.h>

#include "quickfyreapp.h"
#include "quickfyredlgclient.h"
#include "MailingDialog.h"
#include "AboutQuickFyreDialog.h"


DEFINE_RESPONSE_TABLE1(TQuickFyreDlgClient, TDialog)
//{{TQuickFyreDlgClientRSP_TBL_BEGIN}}
  EV_BN_CLICKED(IDOK, MailClicked),
  EV_BN_CLICKED(IDC_ABOUTBUTT, AboutClicked),
  EV_WM_SHOWWINDOW,
  EV_WM_PAINT,
//{{TQuickFyreDlgClientRSP_TBL_END}}
END_RESPONSE_TABLE;


//{{TQuickFyreDlgClient Implementation}}


TQuickFyreDlgClient::TQuickFyreDlgClient(TWindow* parent, TResId resId, TModule* module)
:
  TDialog(parent, resId, module)
{
  // INSERT>> Your constructor code here.
 SenderEdit   = new TEdit(this, IDC_SENDEREDIT);
 ReceiverEdit = new TEdit(this, IDC_TARGETEDIT);
 SubjectEdit  = new TEdit(this, IDC_SUBJECTEDIT);
 ServerEdit   = new TEdit(this, IDC_SERVEREDIT); 
 CopiesEdit   = new TEdit(this, IDC_COPIESEDIT);
 TextEdit     = new TEdit(this, IDC_MAILTEXT);
 CopiesUpDown = new TUpDown(this, IDC_COPIESUPDOWN);

 if(access("qf.dat", 00) == 0)
  {
   QuickFyreDIB = new TDib("qf.dat");
   QFPal = new TPalette(*QuickFyreDIB);
  }
 else
  {
   QuickFyreDIB = NULL;
   MessageBox("Couldn't find qf.dat!\n\
     QuickFyre graphic will not be displayed.\n\
     - Please get the COMPLETE version of QuickFyre. -",
    "Awww man!", MB_OK | MB_ICONEXCLAMATION);
  }  
}


TQuickFyreDlgClient::~TQuickFyreDlgClient()
{
 Destroy();

 // INSERT>> Your destructor code here.
 delete SenderEdit;
 delete SubjectEdit;
 delete ReceiverEdit;
 delete CopiesEdit;
 delete ServerEdit;
 delete TextEdit; 
 delete CopiesUpDown;

 if(QuickFyreDIB != NULL)
  {
   delete QuickFyreDIB;
   delete QFPal;
  }
}

void TQuickFyreDlgClient::MailClicked()
{
  // INSERT>> Your code here.
 char Message[50];

 if(CheckForInputErrors(Message))
  {
   MessageBox(Message, "Error!", MB_OK | MB_ICONEXCLAMATION);
  }
 else 
  {
   MailStruct MailDetails;
   char CopiesText[20];
   
   SenderEdit->GetText(MailDetails.Sender, 70);
   SubjectEdit->GetText(MailDetails.Subject, 70);
   ReceiverEdit->GetText(MailDetails.Receiver, 70);

   CopiesEdit->GetText(CopiesText, 20);
   MailDetails.Copies = atoi(CopiesText);
   
   ServerEdit->GetText(MailDetails.Server, 70);
   TextEdit->GetText(MailDetails.Text, 700); 
   
   TMailingDialog(this, MailDetails).Execute();
  }
}


void TQuickFyreDlgClient::AboutClicked()
{
  // INSERT>> Your code here.
 TAboutQuickFyreDialog(this).Execute();
}


int TQuickFyreDlgClient::EvCreate(CREATESTRUCT far& createStruct)
{
  int result;

  result = TDialog::EvCreate(createStruct);

  // INSERT>> Your code here.
    
  return result;
}


void TQuickFyreDlgClient::EvShowWindow(bool show, uint status)
{
    TDialog::EvShowWindow(show, status);

  // INSERT>> Your code here.
 
}

int TQuickFyreDlgClient::CheckForInputErrors(char *message)
{
 char CurrText[500] ="\0";
 
 SenderEdit->GetText(CurrText, 500);

 if(strcmp(CurrText, "") == 0)
  {
   strcpy(message, "Sender field empty!");
   return 1;
  }

 SubjectEdit->GetText(CurrText, 500);

 if(strcmp(CurrText, "") == 0)
  {
   strcpy(message, "Subject field empty!");
   return 2;
  }

 ReceiverEdit->GetText(CurrText, 500);

 if(strcmp(CurrText, "") == 0)
  {
   strcpy(message, "Reciever field empty!");
   return 3;
  }

 CopiesEdit->GetText(CurrText, 500);

 if(strcmp(CurrText, "") == 0)
  {
   strcpy(message, "Copies field empty!");
   return 4;
  }

  if(atoi(CurrText) == 0)
   {
    strcpy(message, "Invalid copies value!");
    return 5;
   }

 ServerEdit->GetText(CurrText, 500);

 if(strcmp(CurrText, "") == 0)
  {
   strcpy(message, "Server field empty.");
   return 6;
  }
 
 TextEdit->GetText(CurrText, 500);

 if(strcmp(CurrText, "") == 0)
  {
   strcpy(message, "No message!");
   return 7;
  } 

 return 0;
}


void TQuickFyreDlgClient::SetupWindow()
{
  TDialog::SetupWindow();

  // INSERT>> Your code here.
  CopiesUpDown->SetRange(1, UD_MAXVAL);
  CopiesUpDown->SetBuddy(CopiesEdit->HWindow);
}

void TQuickFyreDlgClient::EvPaint()
{
  TDialog::EvPaint();

  // INSERT>> Your code here.
 if(QuickFyreDIB != NULL)
  {
   TClientDC dc(HWindow);
   dc.SelectObject(*QFPal);
   dc.RealizePalette();

   TMemoryDC MemDC;
   TBitmap QFBit(*QuickFyreDIB, QFPal);
   MemDC.SelectObject(QFBit);

   dc.BitBlt(9, 10, QFBit.Width(), QFBit.Height(), MemDC, 0, 0, SRCCOPY);
  } 
}

