unit Sound;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, Volume;

type
  TSoundForm = class(TForm)
    WaveLabel: TLabel;
    WaveBar: TTrackBar;
    CloseButton: TButton;
    RecordButton: TButton;
    RecordLabel1: TLabel;
    LengthEdit: TEdit;
    RecordLabel2: TLabel;
    RecordLabel3: TLabel;
    ProgressLabel: TLabel;
    VolumeControl1: TVolumeControl;
    SynthBar: TTrackBar;
    CDBar: TTrackBar;
    SynthLabel: TLabel;
    CDLabel: TLabel;
    procedure CloseButtonClick(Sender: TObject);
    procedure WaveBarChange(Sender: TObject);
    procedure RecordButtonClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    LastValue : array[0..2] of Integer;
    BarsUpdated : Boolean;
  public
    procedure ResponseInfo(Info : Byte);
    procedure UpdateBars(Wave, Synth, CD : Byte);
  end;

var
  SoundForm: TSoundForm;

implementation

uses Main;

{$R *.DFM}

procedure TSoundForm.ResponseInfo(Info : Byte);
begin
  case Info of
    0: begin
         ProgressLabel.Caption := 'Error';
         RecordButton.Enabled := True;
       end;
    1: ProgressLabel.Caption := 'Recording...';
    2: with MainForm do begin
         // Start download recorded WAVE file.
         TransfFile := '';
         TransfMode := tmWave;
         ClientSock2.Port := DataPort;
         SetAddress(ClientSock2, CurrAddr);
         ClientSock2.Open;
       end;
    3: begin
         ProgressLabel.Caption := 'Done!';
         RecordButton.Enabled := True;
         with MainForm do begin
           SetMediaPlayer(AppPath + 'temp1.wav');
           MediaPlayer1.Play;
         end;
       end;
  end;
end;

procedure TSoundForm.UpdateBars(Wave, Synth, CD : Byte);
begin
  WaveBar.Position := Wave;
  SynthBar.Position := Synth;
  CDBar.Position := CD;
  LastValue[0] := Wave;
  LastValue[1] := Synth;
  LastValue[2] := CD;
  BarsUpdated := True;
end;

procedure TSoundForm.CloseButtonClick(Sender: TObject);
begin
  Close;
end;

procedure TSoundForm.WaveBarChange(Sender: TObject);
var
  DevType : Integer;
  CurrPos : Integer;
begin
  with Sender as TTrackBar do begin
    DevType := Tag;
    CurrPos := Position;
  end;
  if not BarsUpdated or (LastValue[DevType] = CurrPos) then
    Exit;
  LastValue[DevType] := CurrPos;
  with MainForm do begin
    if RemoteVer <= '1.50' then
      ClientText := 'SetVolume;' + IntToStr(LastValue[0]) + #13
    else ClientText := 'SetVolume;' + IntToStr(DevType) + ';' +
      IntToStr(CurrPos) + #13;
  end;
end;

procedure TSoundForm.RecordButtonClick(Sender: TObject);
begin
  if MainForm.RemoteVer <= '1.20' then begin
    Application.MessageBox('Uppgradera till en nyare version hos vrden.',
      'Info', MB_ICONEXCLAMATION or MB_OK);
    Exit;
  end;
  RecordButton.Enabled := False;
  MainForm.ClientText := 'Record;' + LengthEdit.Text + #13;
end;

procedure TSoundForm.FormShow(Sender: TObject);
begin
  BarsUpdated := MainForm.RemoteVer <= '1.50';
  if MainForm.RemoteVer <= '1.50' then
    WaveBar.Max := 100;
  MainForm.ClientText := 'GetVolume'#13;
end;

end.
