unit Wave;

interface

procedure CreateWav(channels : word; resolution : word;    { 8 or 16     }
  rate : longint; fn : string);

implementation

type
 { format of WAV file header }
 TWavHeader = record         { parameter description }
   rId             : longint; { 'RIFF'  4 characters }
   rLen            : longint; { length of DATA + FORMAT chunk }
   { FORMAT CHUNK }
   wId             : longint; { 'WAVE' }
   fId             : longint; { 'fmt ' }
   fLen            : longint; { length of FORMAT DATA = 16 }
   { format data }
   wFormatTag      : word;    { $01 = PCM }
   nChannels       : word;    { 1 = mono, 2 = stereo }
   nSamplesPerSec  : longint; { Sample frequency ie 11025}
   nAvgBytesPerSec : longint; { = nChannels * nSamplesPerSec *
                                 (nBitsPerSample/8) }
   nBlockAlign     : word;    { = nChannels * (nBitsPerSAmple / 8 }
   wBitsPerSample  : word;    { 8 or 16 }
   { DATA CHUNK }
   dId             : longint; { 'data' }
   wSampleLength   : longint; { length of SAMPLE DATA }
   { sample data : offset 44 }
    { for 8 bit mono = s[0],s[1]... :byte}
    { for 8 bit stereo = sleft[0],sright[0],sleft[1],sright[1]... :byte}
    { for 16 bit mono = s[0],s[1]... :word}
    { for 16 bit stereo = sleft[0],sright[0],sleft[1],sright[1]... :word}
  end;

{ here is a routine which will create a blank wav file with
  the specified parameters...
    channels   : 1=mono, 2=stereo
    resolution : 8 or 16 bits
    rate       : sample rate ie 11025, 44100 etc
    fn         : name of new file
}

procedure CreateWav( channels   : word;    { 1 or 2      }
                     resolution : word;    { 8 or 16     }
                     rate       : longint; { sample rate }
                     fn         : string   { file name   } );
var
  wf : file of TWavHeader;
  wh : TWavHeader;
begin
  wh.rId             := $46464952; { 'RIFF' }
  wh.rLen            := 36;        { length of sample + format }
  wh.wId             := $45564157; { 'WAVE' }
  wh.fId             := $20746d66; { 'fmt ' }
  wh.fLen            := 16;        { length of format chunk }
  wh.wFormatTag      := 1;         { PCM data }
  wh.nChannels       := channels;  { mono/stereo }
  wh.nSamplesPerSec  := rate;      { sample rate }
  wh.nAvgBytesPerSec := channels*rate*(resolution div 8);
  wh.nBlockAlign     := channels*(resolution div 8);
  wh.wBitsPerSample  := resolution;{ resolution 8/16 }
  wh.dId             := $61746164; { 'data' }
  wh.wSampleLength   := 0;         { sample size }

  assignfile(wf,fn);               { assign file }
  rewrite(wf);                     { open file for writing }
  write(wf,wh);                    { write header }
  closefile(wf);                   { close file }
end;

end.
